package common

import (
	"fmt"
	"strconv"
	"strings"

	"github.com/gofrs/uuid"
)

func IntRange(min, max int) []int {
	a := make([]int, max-min+1)
	for i := range a {
		a[i] = min + i
	}
	return a
}

func StringIntRange(min, max int) []string {
	results := make([]string, max-min+1)
	for i, v := range IntRange(min, max) {
		results[i] = strconv.Itoa(v)
	}
	return results
}

func MapStrings(in []string, transform func(string) string) []string {
	results := make([]string, len(in))
	for i, v := range in {
		results[i] = transform(v)
	}
	return results
}

func AddPrefix(prefix string) func(string) string {
	return func(in string) string {
		return fmt.Sprintf("%s%s", prefix, in)
	}
}

func Dollar(in string) string {
	return fmt.Sprintf("$%s", in)
}

func DollarForEach(max int) []string {
	return MapStrings(StringIntRange(1, max), Dollar)
}

func StringsToInterfaces(input []string) []interface{} {
	results := make([]interface{}, len(input))
	for i, value := range input {
		results[i] = value
	}
	return results
}

func UUIDsToInterfaces(input []uuid.UUID) []interface{} {
	results := make([]interface{}, len(input))
	for i, value := range input {
		results[i] = value.String()
	}
	return results
}

func RemoveTrimmedEmptyStrings(input []string) []string {
	results := make([]string, 0)
	for _, s := range input {
		if trimmed := strings.TrimSpace(s); len(trimmed) > 0 {
			results = append(results, trimmed)
		}
	}
	return results
}

func StringToUUIDMapValues(thing map[string]uuid.UUID) []uuid.UUID {
	results := make([]uuid.UUID, 0, len(thing))
	for _, value := range thing {
		results = append(results, value)
	}
	return results
}
