package storage

import (
	"github.com/ngerakines/tavern/common"
)

func JSONDeepString(document map[string]interface{}, keys ...string) (string, bool) {
	if len(keys) == 1 {
		return JSONString(document, keys[0])
	}
	if len(keys) > 1 {
		inner, ok := JSONMap(document, keys[0])
		if ok {
			return JSONDeepString(inner, keys[1:]...)
		}
	}
	return "", false
}

func CollectJSONDeepStrings(document map[string]interface{}, keySets ...[]string) []string {
	results := make([]string, 0)
	for _, keys := range keySets {
		values, _ := JSONDeepStrings(document, keys...)
		results = append(results, values...)
	}
	return common.RemoveTrimmedEmptyStrings(results)
}

func FirstJSONDeepStrings(document map[string]interface{}, keySets ...[]string) string {
	results := make([]string, 0)
	for _, keys := range keySets {
		values, _ := JSONDeepStrings(document, keys...)
		results = append(results, values...)
	}
	values := common.RemoveTrimmedEmptyStrings(results)
	if len(values) == 0 {
		return ""
	}
	return values[0]
}

func JSONDeepStrings(document map[string]interface{}, keys ...string) ([]string, bool) {
	if len(keys) == 1 {
		return JSONStrings(document, keys[0])
	}
	if len(keys) > 1 {
		inner, ok := JSONMap(document, keys[0])
		if ok {
			return JSONDeepStrings(inner, keys[1:]...)
		}
	}
	return []string{}, false
}

func JSONMap(document map[string]interface{}, key string) (map[string]interface{}, bool) {
	if value, ok := document[key]; ok {
		if mapVal, isMap := value.(map[string]interface{}); isMap {
			return mapVal, true
		}
	}
	return nil, false
}

func JSONString(document map[string]interface{}, key string) (string, bool) {
	if value, ok := document[key]; ok {
		if strValue, isString := value.(string); isString {
			return strValue, true
		}
	}
	return "", false
}

func JSONStrings(document map[string]interface{}, key string) ([]string, bool) {
	results := make([]string, 0)
	value, ok := document[key]
	if !ok {
		return nil, false
	}
	switch v := value.(type) {
	case string:
		results = append(results, v)
	case []interface{}:
		for _, el := range v {
			if strValue, isString := el.(string); isString {
				results = append(results, strValue)
			}
		}
	case []string:
		for _, el := range v {
			results = append(results, el)
		}
	}
	return results, true
}

func JSONMapList(document map[string]interface{}, key string) ([]map[string]interface{}, bool) {
	results := make([]map[string]interface{}, 0)
	value, ok := document[key]
	if !ok {
		return nil, false
	}
	switch v := value.(type) {
	case map[string]interface{}:
		results = append(results, v)
	case []interface{}:
		for _, el := range v {
			if mapValue, isMap := el.(map[string]interface{}); isMap {
				results = append(results, mapValue)
			}
		}
	}
	return results, true
}

func StringsContainsString(things []string, value string) bool {
	for _, thing := range things {
		if thing == value {
			return true
		}
	}
	return false
}
