package web

import (
	"math"
	"net/url"
	"strconv"
)

type paged struct {
	Page      int
	Pages     []pagedPage
	Total     int
	Limit     int
	Location  string
	PageCount float64
}

type pagedPage struct {
	Current bool
	Label   string
	URL     string
}

func createPaged(limit, currentPage, total int, location string) (paged, error) {
	pageCount := math.Ceil(float64(total) / float64(limit))

	u, err := url.Parse(location)
	if err != nil {
		return paged{}, err
	}

	var pagedPages []pagedPage

	for i := currentPage - 3; i <= currentPage+3; i++ {
		if i > 0 && i <= int(pageCount) {
			q := u.Query()
			q.Set("page", strconv.Itoa(i))
			u.RawQuery = q.Encode()
			pagedPages = append(pagedPages, pagedPage{
				Current: i == currentPage,
				Label:   strconv.Itoa(i),
				URL:     u.String(),
			})
		}
	}

	return paged{
		Location:  location,
		Page:      currentPage,
		PageCount: pageCount,
		Pages:     pagedPages,
		Limit:     limit,
		Total:     total,
	}, nil
}
